import requests

from license_service._environment import Environment
from license_service.exceptions import InvalidFeatureException, InvalidLicenseException


class LicenseValidationService:
    _license_key = Environment.get("LICENSE_KEY", str, None)
    _license_server_api_key = Environment.get(
        "LICENSE_SERVER_API_KEY",
        str,
        "api_hwsdCpcLV9ukgy-edGdO5z8f6CB13EVc5cKokdgahyhkI2SMNsBef5sk7odYNTqrF-lHNJPLZK7k4PYQSMf_ww"
    )  # default validation key
    _license_server_url = Environment.get(
        "LICENSE_SERVER_URL",
        str,
        "https://license.sh-edraft.de/"
    )

    @classmethod
    def _http_validate_license(cls, product_name: str) -> bool:
        if cls._license_key is None:
            raise InvalidLicenseException(
                "No license key provided. Please set the LICENSE_KEY environment variable."
            )

        response = requests.get(f"{cls._license_server_url}/api/validate/{cls._license_key}/{product_name}", headers={
            "Authorization": f"API-Key {cls._license_server_api_key}"
        })

        # Check the status code first
        if response.status_code == 200:
            return response.text.strip().lower() == 'true'

        raise ConnectionError(response.status_code, response.text)

    @classmethod
    def _http_has_feature(cls, license_key: str, name: str, count: int = None) -> bool:
        url = f"{cls._license_server_url}/api/validate/{license_key}/has-feature/{name}"
        if count is not None:
            url += f"/{count}"

        response = requests.get(url, headers={
            "Authorization": f"API-Key {cls._license_server_api_key}"
        })

        # Check the status code first
        if response.status_code == 200:
            return response.text.strip().lower() == 'true'

        raise ConnectionError(response.status_code, response.text)

    @classmethod
    def check_license_by_environment(cls):
        if not cls._http_validate_license("open-redirect"):
            raise InvalidLicenseException(
                f"License key '{cls._license_key}' is not valid. "
                "Please check your license key or contact support."
            )

    @classmethod
    def has_feature(cls, name: str, count: int = None) -> bool:
        return cls._http_has_feature(cls._license_key, name, count)

    @classmethod
    def has_license_feature(cls, license_key: str, name: str, count: int = None) -> bool:
        return cls._http_has_feature(license_key, name, count)

    @classmethod
    def check_has_feature(cls, name: str, count: int = None) -> bool:
        if not cls._http_has_feature(cls._license_key, name, count):
            raise InvalidFeatureException(
                name,
                f"Feature '{name}' is not available with the current license'. "
                "Please check your license or contact support."
            )
        return True

    @classmethod
    def check_license_has_feature(cls, license_key: str, name: str, count: int = None) -> bool:
        if not cls._http_has_feature(license_key, name, count):
            raise InvalidFeatureException(
                name,
                f"Feature '{name}' is not available with the current license'. "
                "Please check your license or contact support."
            )
        return True
